@@ -586,6 +586,21 @@ gupnp_service_action_return_error (GUPnPServiceAction *action,
         finalize_action (action);
 }
 
+/**
+ * gupnp_service_action_get_message
+ * @action: A #GUPnPServiceAction
+ *
+ * Get the #SoupMessage associated with @action. Mainly intended for
+ * applications to be able to read HTTP headers received from clients.
+ *
+ * Return value: #SoupMessage associated with @action. Unref after using it.
+ **/
+SoupMessage *
+gupnp_service_action_get_message (GUPnPServiceAction *action)
+{
+        return g_object_ref (action->msg);
+}
+
 static void
 gupnp_service_init (GUPnPService *service)
 {
@@ -604,11 +619,10 @@ gupnp_service_init (GUPnPService *service)
 
 /* Generate a new action response node for @action_name */
 static GString *
-new_action_response_str (GUPnPService *service,
-                         const char   *action_name)
+new_action_response_str (const char   *action_name,
+                         const char   *service_type)
 {
         GString *str;
-        const char *type;
 
         str = xml_util_new_string ();
 
@@ -616,11 +630,8 @@ new_action_response_str (GUPnPService *service,
         g_string_append (str, action_name);
         g_string_append (str, "Response xmlns:u=");
 
-        type = gupnp_service_info_get_service_type
-                                        (GUPNP_SERVICE_INFO (service));
-        if (type != NULL) {
-                g_string_append_c (str, '"');
-                g_string_append (str, type);
+        if (service_type != NULL) {
+                g_string_append (str, service_type);
                 g_string_append_c (str, '"');
         } else {
                 g_warning ("No serviceType defined. Control may not work "
@@ -701,9 +712,11 @@ control_server_handler (SoupServer        *server,
         GUPnPContext *context;
         xmlDoc *doc;
         xmlNode *action_node;
-        const char *soap_action, *action_name;
+        const char *soap_action;
+        char *action_name;
         char *end;
         GUPnPServiceAction *action;
+        goffset length;
 
         service = GUPNP_SERVICE (user_data);
 
@@ -713,11 +726,23 @@ control_server_handler (SoupServer        *server,
                 return;
         }
 
+        length = soup_message_headers_get_content_length (msg->request_headers);
+        if (length == 0) {
+                soup_message_set_status (msg, SOUP_STATUS_BAD_REQUEST);
+
+                return;
+        }
+
         context = gupnp_service_info_get_context (GUPNP_SERVICE_INFO (service));
 
         /* Get action name */
         soap_action = soup_message_headers_get (msg->request_headers,
                                                 "SOAPAction");
+        if (!soap_action) {
+                soup_message_set_status (msg, SOUP_STATUS_PRECONDITION_FAILED);
+                return;
+        }
+
         action_name = strchr (soap_action, '#');
         if (!action_name) {
                 soup_message_set_status (msg, SOUP_STATUS_PRECONDITION_FAILED);
@@ -725,15 +750,24 @@ control_server_handler (SoupServer        *server,
                 return;
         }
 
+        /* This memory is libsoup-owned so we can do this */
+        *action_name = '\0';
         action_name += 1;
 
         /* This memory is libsoup-owned so we can do this */
         end = strrchr (action_name, '"');
-        *end = '\0';
+        if (end)
+                *end = '\0';
 
         /* Parse action_node */
         doc = xmlRecoverMemory (msg->request_body->data,
                                 msg->request_body->length);
+        if (doc == NULL) {
+                soup_message_set_status (msg, SOUP_STATUS_BAD_REQUEST);
+
+                return;
+        }
+
         action_node = xml_util_get_element ((xmlNode *) doc,
                                             "Envelope",
                                             "Body",
@@ -752,7 +786,8 @@ control_server_handler (SoupServer        *server,
         action->msg          = g_object_ref (msg);
         action->doc          = doc;
         action->node         = action_node;
-        action->response_str = new_action_response_str (service, action_name);
+        action->response_str = new_action_response_str (action_name,
+                                                        soap_action);
         action->context      = g_object_ref (context);
 
         /* Tell soup server that response is not ready yet */
